/*
   Copyright (C) 2007 - 2009 by Roberto García Carvajal <roberpot@gmail.com>
   Part of libglsws (http://magiccards.wordpress.com)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3
   or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/

#include <GL/gl.h>
#include <string>
#include <fstream>
#include <deque>
#include <map>
#include <set>

using namespace std;

#ifndef _C_LEXICO_
#define _C_LEXICO_


/*******************************************************************************
*
* Definiciones de los Tokens.
*
*******************************************************************************/
#define L_FIN         0

#define L_VENTANA     256

#define L_ID          257
#define L_X           258
#define L_Y           259
#define L_IMG         260
#define L_ANCHO       261
#define L_ALTO        262
#define L_VISIBLE     263
#define L_MOVIL       264
#define L_COLOR       265
#define L_BCOLOR      266
#define L_TCOLOR      267
#define L_TEXTO       268
#define L_FACTORTEXTO 269
#define L_GROSORTEXTO 270

#define L_OBJETO            500
#define L_OBOTON            501
#define L_OIMAGEN           502
#define L_OTEXTO            503
#define L_OCASILLERO        504
#define L_OTEXTOEDITABLE    505
#define L_OSELECTOR         506
#define L_OSELECTORFICHEROS 507

#define L_ENTERO   600
#define L_FLOTANTE 601
#define L_CADENA   602
#define L_TRUE     603
#define L_FALSE    604

#define L_ECLICK   700
#define L_EBLUR    701

#define L_RETORNO    800
#define L_PARAMETROS 801

#define L_ERROR    5000

/*******************************************************************************
*
* Clase c_lexico
*
* Analizador léxico.
*
*******************************************************************************/
class c_lexico
{
 public:
  /// Constructor. Recibe un nombre de fichero.
  c_lexico(const GLbyte * f);
  /// El analizador léxico lee el siguiente token.
  void leer();
  /// Devuelve el token actual.
  GLuint token();
  /// Devuelve el lexema (si es de tipo entero).
  GLuint ilexema(){return _i_lexema;}
  /// Devuelve el lexema (si es de tipo cadena).
  string slexema(){return _s_lexema;}
  /// Devuelve el lexema (si es de tipo flotante).
  GLfloat flexema(){return _f_lexema;}
  /// Devuelve un token.
  void rebobinar();
 private:
  /// Lectura de una cadena.
  bool _leer_cadena();
  /// Lectura de un número.
  bool _leer_numero();
  /// Lectura de una cadena (dato).
  bool _leer_dato_cadena();
  /// Token actual.
  GLuint _token;
  /// Fichero de entrada.
  ifstream _fichero;
  /// Lexema de tipo entero.
  GLuint _i_lexema;
  /// Lexema de tipo flotante.
  GLfloat _f_lexema;
  /// Lexema de tipo cadena.
  string _s_lexema;
  /// Cola doble para poder rebobinar.
  deque<GLuint> _cinta;
  /// Para mapear las cadenas.
  map<string,GLuint> _mapa_cadenas;
  /// Carácteres que también son tokens.
  set<GLubyte> _conjunto_caracteres;
};

#endif
