/*
   Copyright (C) 2007 - 2009 by Roberto García Carvajal <roberpot@gmail.com>
   Part of libglsws (http://magiccards.wordpress.com)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3
   or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/

#include <GL/gl.h>
#include <SDL/SDL.h>
#include <FTGL/ftgl.h>
#include <string>
#include <ctime>
#include <boost/filesystem.hpp>
#include <vector>
#include "c_esdatos.h"

using namespace std;

#ifndef _C_OVENTANA_
#define _C_OVENTANA_


/*******************************************************************************
*
* Definiciones de los ids de los objetos.
*
*******************************************************************************/
#define O_T_BOTON            1
#define O_T_TEXTO            2
#define O_T_CASILLERO        3
#define O_T_TEXTOEDITABLE    4
#define O_T_SELECTOR         5
#define O_T_SELECTORFICHEROS 6

/*******************************************************************************
*
* Definiciones de las banderas de los objetos.
*
*******************************************************************************/
#define O_VISIBLE            0x01
#define O_TEXTURADO          0x02
#define O_ACTIVO             0x04

/*******************************************************************************
*
* Clase c_evento
*
* Clase para los eventos de los objetos de las ventanas.
*
*******************************************************************************/
struct c_parametro
{
 string s;
 GLuint i,t_parametro;
 GLfloat f,f3[3];
 bool b;
};

/*******************************************************************************
*
* Clase c_accion
*
* Clase para los eventos de los objetos de las ventanas.
*
*******************************************************************************/
struct c_accion
{
 GLuint ventana;
 GLuint objeto;
 bool retorno;
 vector<c_parametro> parametros;
};

/*******************************************************************************
*
* Clase c_oventana
*
* Objeto de ventana genérico, del que heredarán el resto de objetos.
*
*******************************************************************************/
class c_oventana
{
 public:
  /// Constructor.
  c_oventana();
  /// Destructor (virtual).
  virtual ~c_oventana() {}
  /// Inicializa la fuente.
  void fuente(FTGLTextureFont * f) { _fuente = f; }
  /// Devuelve el tipo del objeto.
  GLuint tipo() { return _o_tipo; }
  /// Para la posición horizontal.
  void x(GLfloat x) { _o_x = x; }
  /// Para la posición vertical.
  void y(GLfloat y) { _o_y = y; }
  /// Para el alto.
  void alto(GLfloat a) { _o_alto = a; }
  /// Para el ancho.
  void ancho(GLfloat a) { _o_ancho = a; }
  /// Para el texto.
  void texto(string t) { _o_texto = t; }
  /// Instala banderas.
  void set(GLuint BE) { _RBE |= BE; }
  /// Retira banderas.
  void unset(GLuint BE) { _RBE &= ~BE; }
  /// Para la textura.
  void textura(GLuint tex) { _o_textura = tex; }
  /// Para el color de fondo.
  void colorf(GLfloat c[3]) { _o_colorf[0] = c[0]; _o_colorf[1] = c[1]; _o_colorf[2] = c[2]; }
  /// Para el color del borde.
  void colorb(GLfloat c[3]) { _o_colorb[0] = c[0]; _o_colorb[1] = c[1]; _o_colorb[2] = c[2]; }
  /// Para el color del texto.
  void colort(GLfloat c[3]) { _o_colort[0] = c[0]; _o_colort[1] = c[1]; _o_colort[2] = c[2]; }
  /// Para el tamaño del texto.
  void textoTamano(GLfloat tam) { _o_textoTamano = tam; }
  /// Para el grueso del texto (NO USAR).
  void textoGrosor(GLfloat grosor) { _o_textoGrosor = grosor; }
  /// Añade un carácter al texto.
  void addc(GLbyte c);
  /// Borra un carácter del texto.
  void remc();
  /// Hace un chequeo para comprobar la integridad interna.
  void chequear();
  /// cargar Virtual, para que cada clase que hereda haga su propia interpretación.
  virtual void cargar(c_dato d) {}
  /// guardar Virtual, para que cada clase que hereda haga su propia devolución.
  virtual c_dato guardar() { c_dato d; return d; }
  /// Renderizar el objeto. Virtual evidentemente.
  virtual void dibujar(GLfloat x,GLfloat y) {}
  /// Para determinar si se ha hecho click dentro del objeto o no.
  bool click(GLfloat x,GLfloat y);
  /// Para añadir acciones asociadas a eventos.
  void nuevaaccion(GLuint evento, c_accion accion);
  /// Para devolver acciones asociadas a un evento.
  vector<c_accion> acciones(GLuint evento);
 protected:
  /// Postchequeo, virtual, para que cada objeto que herede de este pueda definir
  /// sus propios chequeos internos.
  virtual void postchequeo() {}
  /// Para cargar un dato externo. Virtual puesto que cada objeto que herede
  /// puede hacer una gestión distinta con el dato.
  virtual void ejecutar_acciones(GLfloat x, GLfloat y) {}
  GLuint _RBE ,                         /// Banderas de estado.
         _o_tipo ,                      /// Tipo del objeto.
         _o_textura ,                   /// Textura del objeto.
         _o_textoLongitudMaxima,        /// Longitud máxima del texto (deprecated?)
         _o_posicionCentradaHorizontal, /// Posición centrada horizontal.
         _o_posicionCentradaVertical;   /// Posición centrada vertical.
  GLfloat _o_x ,           /// Posición horizontal.
          _o_y ,           /// Posición vertical.
          _o_ancho ,       /// Ancho.
          _o_alto ,        /// Alto.
          _o_textoTamano , /// Tamaño del texto.
          _o_textoGrosor , /// Grosor del texto (deprecated).
          _o_colorf[3] ,   /// Color de fondo.
          _o_colorb[3] ,   /// Color de borde.
          _o_colort[3];    /// Color de texto.
  string _o_texto;         /// Texto.
  FTGLTextureFont * _fuente; /// Fuente.
  map<GLuint,vector<c_accion> > _eventos; /// Para los eventos.
};

/*******************************************************************************
*
* Clase c_oboton
*
* Botones para las ventanas.
*
*******************************************************************************/
class c_oboton : public c_oventana
{
 public:
  /// Constructor.
  c_oboton(GLuint rbe = O_VISIBLE | O_ACTIVO);
  /// Renderizar.
  void dibujar(GLfloat x,GLfloat y);
  /// Devolver dato.
  c_dato guardar();
  /// Destructor.
  ~c_oboton();
};

/*******************************************************************************
*
* Clase c_otexto
*
* Texto para las ventanas.
*
*******************************************************************************/
class c_otexto : public c_oventana
{
 public:
  /// Constructor.
  c_otexto(GLuint rbe = O_VISIBLE | O_ACTIVO);
  /// Sobrecargo el método para cargar datos.
  void cargar(c_dato d);
  /// Método para renderizar el texto.
  void dibujar(GLfloat x,GLfloat y);
  /// Devolver dato.
  c_dato guardar();
  /// Destructor.
  ~c_otexto();
};

/*******************************************************************************
*
* Clase c_ocasillero
*
* Casillero (checkbox)
*
*******************************************************************************/
class c_ocasillero : public c_oventana
{
 public:
  /// Constructor.
  c_ocasillero(GLuint rbe = O_VISIBLE | O_ACTIVO);
  /// Para renderizar el casillero.
  void dibujar(GLfloat x,GLfloat y);
  /// Sobrecargo el método para cargar datos.
  void cargar(c_dato d);
  /// Devolver dato.
  c_dato guardar();
  /// Destructor.
  ~c_ocasillero();
 private:
  /// Sobrecargo el método para ejecutar acciones.
  void ejecutar_acciones(GLfloat x, GLfloat y);
  /// Dato booleano para saber si está seleccionado.
  bool _seleccionado;
};

/*******************************************************************************
*
* Clase c_otextoeditable
*
* Cuadros de texto de las ventanas.
*
*******************************************************************************/
class c_otextoeditable : public c_oventana
{
 public:
  /// Constructor
  c_otextoeditable(GLuint rbe = O_VISIBLE | O_ACTIVO);
  /// Sobreescribo el método que lo renderiza.
  void dibujar(GLfloat x,GLfloat y);
  /// Sobreescribo el método que carga datos.
  void cargar(c_dato d);
  /// Devolver dato.
  c_dato guardar();
  /// Destructor.
  ~c_otextoeditable();
 private:
  /// Sobreescribo el método que ejecuta acciones.
  void ejecutar_acciones() {}
};

/*******************************************************************************
*
* Clase c_oselector
*
* Un selector de opciones.
*
*******************************************************************************/
class c_oselector : public c_oventana
{
 public:
  /// Constructor.
  c_oselector(GLuint rbe = O_VISIBLE | O_ACTIVO);
  /// Sobrecargo el método que renderiza.
  void dibujar(GLfloat x,GLfloat y);
  /// Sobrecargo el método que carga datos.
  void cargar(c_dato d);
  /// Devolver dato.
  c_dato guardar();
  /// Destructor.
  ~c_oselector();
 private:
  /// Sobrecargo el método que hace el postchequeo, este objeto necesita revisar
  /// más variables.
  void postchequeo();
  /// Sobrecargo el método que ejecuta acciones.
  void ejecutar_acciones(GLfloat x, GLfloat y);
  GLuint _barra_x_1,     /// Para la barra de movimiento.
         _barra_x_2,     /// Para la barra de movimiento.
         _barra_y_1,     /// Para la barra de movimiento.
         _barra_y_2,     /// Para la barra de movimiento.
         _seleccionado,  /// Opcion seleccionada.
         _posactual,     /// Primer elemento visible.
         _longtotal,     /// Número de elementos que caben en la lista a la vez.
         _talto,         /// Alto de cada elemento.
         _tsalto;        /// Para el recuadro de selección.
  vector<pair<GLint,string> > lista; /// Lista de opciones.
};

/*******************************************************************************
*
* Clase c_oselectorficheros
*
* Un navegador por el sistema de ficheros.
*
*******************************************************************************/
class c_oselectorficheros : public c_oventana
{
 public:
  /// Constructor.
  c_oselectorficheros(GLuint rbe = O_VISIBLE | O_ACTIVO);
  /// Sobrecargo el método para renderizarlo.
  void dibujar(GLfloat x,GLfloat y);
  /// Sobrecargo el método para cargar datos.
  void cargar(c_dato d);
  /// Devolver dato.
  c_dato guardar();
  /// Destructor.
  ~c_oselectorficheros();
 private:
  /// Sobrecargo el método para el chequeo interno.
  void postchequeo();
  /// Sobrecargo el método para ejecutar acciones post-click.
  void ejecutar_acciones(GLfloat x, GLfloat y);
  /// Método para recargar la lista de ficheros.
  void recargar_lista();
  GLuint _barra_x_1,     /// Para la barra de movimiento.
         _barra_x_2,     /// Para la barra de movimiento.
         _barra_y_1,     /// Para la barra de movimiento.
         _barra_y_2,     /// Para la barra de movimiento.
         _seleccionado,  /// Opcion seleccionada.
         _posactual,     /// Primer elemento visible.
         _longtotal,     /// Número de elementos que caben en la lista a la vez.
         _talto,         /// Alto de cada elemento.
         _tsalto,        /// Para el recuadro de selección.
         _posclic;       /// Posición donde se hizo el último click.
  clock_t _tclic;        /// Tiempo en que se hizo el último click.
  boost::filesystem::path fpath;     /// Directorio actual.
  vector<pair<GLint,pair<string,string> > > directorio;     /// Lista del directorio actual.
};
#endif
