/*
   Copyright (C) 2007 - 2009 by Roberto García Carvajal <roberpot@gmail.com>
   Part of libglsws (http://magiccards.wordpress.com)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3
   or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/

#include <iostream>
#include "c_lexico.h"
#include "c_sintactico.h"

/*********************************************************
*
* Constructores de los nodos.
*
*********************************************************/

nodo_parametro::nodo_parametro()
{
 t_parametro = 0;
 i_valor = 0;
 s_valor = "";
 b_valor = false;
 fv3_valor[0] = 0.0;
 fv3_valor[1] = 0.0;
 fv3_valor[2] = 0.0;
 parametro = 0;
}

nodo_accion::nodo_accion()
{
 accion = 0;
 parametro = 0;
 ventana = 0;
 objeto = 0;
 datos = false;
}

nodo_evento::nodo_evento()
{
 t_evento = 0;
 accion = 0;
 evento = 0;
}

nodo_objeto::nodo_objeto()
{
 t_objeto = 0;
 parametro = 0;
 evento = 0;
 objeto = 0;
}

nodo_ventana::nodo_ventana()
{
 parametro = 0;
 objeto = 0;
}

nodo_ventanas::nodo_ventanas()
{
 ventana = 0;
 ventanas = 0;
}

/*********************************************************
*
* Inicio del analizador sintáctico descendente recursivo.
*
*********************************************************/

/*******************************************************************************
*
* Método _asdr_parametro()
*
* Gramática:
* PARAMETROS   => PARAMETRO , PARAMETROS
*              => PARAMETRO
* PARAMETRO    => PARAMETRO_I
* PARAMETRO    => PARAMETRO_B
* PARAMETRO    => PARAMETRO_F3
* PARAMETRO_I  => I_TOKEN '=' I_LEXEMA
* PARAMETRO_B  => B_TOKEN '=' B_LEXEMA
* PARAMETRO_F3 => F3_TOKEN '=' '[' F_LEXEMA ',' F_LEXEMA ',' F_LEXEMA ']'
*
*******************************************************************************/
nodo_parametro * c_sintactico::_asdr_parametro()
{
 nodo_parametro * nodo = new nodo_parametro();

 _analizador_lexico->leer();
 nodo->t_parametro = _analizador_lexico->token();
 if(_i_tokens.count(_analizador_lexico->token()))
 {
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != '=')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba '='" << std::endl;
   delete nodo;
   return 0;
  }
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != L_ENTERO)
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba un entero" << std::endl;
   delete nodo;
   return 0;
  }
  nodo->i_valor = _analizador_lexico->ilexema();
 }
 else if(_s_tokens.count(_analizador_lexico->token()))
 {
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != '=')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba '='" << std::endl;
   delete nodo;
   return 0;
  }
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != L_CADENA)
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba un entero" << std::endl;
   delete nodo;
   return 0;
  }
  nodo->s_valor = _analizador_lexico->slexema();
 }
 else if(_b_tokens.count(_analizador_lexico->token()))
 {
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != '=')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba '='" << std::endl;
   delete nodo;
   return 0;
  }
  _analizador_lexico->leer();
  if(_analizador_lexico->token() == L_TRUE)
  {
   nodo->b_valor = true;
  }
  else if(_analizador_lexico->token() == L_FALSE)
  {
   nodo->b_valor = false;
  }
  else
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba un entero" << std::endl;
   delete nodo;
   return 0;
  }
 }
 else if(_3f_tokens.count(_analizador_lexico->token()))
 {
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != '=')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba '='" << std::endl;
   delete nodo;
   return 0;
  }
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != '[')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba '['" << std::endl;
   delete nodo;
   return 0;
  }
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != L_FLOTANTE)
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba un flotante" << std::endl;
   delete nodo;
   return 0;
  }
  nodo->fv3_valor[0] = _analizador_lexico->flexema();
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != ',')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba ','" << std::endl;
   delete nodo;
   return 0;
  }
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != L_FLOTANTE)
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba un flotante" << std::endl;
   delete nodo;
   return 0;
  }
  nodo->fv3_valor[1] = _analizador_lexico->flexema();
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != ',')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba ','" << std::endl;
   delete nodo;
   return 0;
  }
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != L_FLOTANTE)
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba un flotante" << std::endl;
   delete nodo;
   return 0;
  }
  nodo->fv3_valor[2] = _analizador_lexico->flexema();
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != ']')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba ']'" << std::endl;
   delete nodo;
   return 0;
  }
 }
 else
 {
  std::cout << "Analizador sintáctico:: Error:: Token inesperado" << std::endl;
  delete nodo;
  return 0;
 }
 _analizador_lexico->leer();
 if(_analizador_lexico->token() == ',')
 {
  nodo->parametro = _asdr_parametro();
 }
 else
 {
  _analizador_lexico->rebobinar();
 }
 return nodo;
}

/*******************************************************************************
*
* Método _asdr_accion()
*
* Gramática:
* ACCIONES => L_VENTANA '[' I_LEXEMA ']' '.' L_OBJETO '[' I_LEXEMA ']' '{'
*             PARAMETROS '}' ACCIONES
*          => L_VENTANA '[' I_LEXEMA ']' '{' PARAMETROS '}' ACCIONES
*          => L_VENTANA '[' I_LEXEMA ']' '.' L_DATOS ACCIONES
*          => *
*
*******************************************************************************/
nodo_accion * c_sintactico::_asdr_accion()
{
 nodo_accion * nodo = new nodo_accion();
 _analizador_lexico->leer();
 if(_analizador_lexico->token() == L_VENTANA)
 {
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != '[')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba '['." << std::endl;
   delete nodo;
   return 0;
  }
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != L_ENTERO)
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba L_ENTERO." << std::endl;
   delete nodo;
   return 0;
  }
  nodo->ventana = _analizador_lexico->ilexema();
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != ']')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba ']'." << std::endl;
   delete nodo;
   return 0;
  }
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != '.')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba '.'." << std::endl;
   delete nodo;
   return 0;
  }
  _analizador_lexico->leer();
  if(_analizador_lexico->token() == L_RETORNO)
  {
   nodo->datos = true;
  }
  else if(_analizador_lexico->token() == L_PARAMETROS)
  {
   _analizador_lexico->leer();
   if(_analizador_lexico->token() != '{')
   {
    std::cout << "Analizador sintáctico:: Error:: Se esperaba '{'." << std::endl;
    delete nodo;
    return 0;
   }
   nodo->parametro = _asdr_parametro();
   _analizador_lexico->leer();
   if(_analizador_lexico->token() != '}')
   {
    std::cout << "Analizador sintáctico:: Error:: Se esperaba '}'." << std::endl;
    delete nodo;
    return 0;
   }
  }
  else if(_analizador_lexico->token() == L_OBJETO)
  {
   _analizador_lexico->leer();
   if(_analizador_lexico->token() != '[')
   {
    std::cout << "Analizador sintáctico:: Error:: Se esperaba '['." << std::endl;
    delete nodo;
    return 0;
   }
   _analizador_lexico->leer();
   if(_analizador_lexico->token() != L_ENTERO)
   {
    std::cout << "Analizador sintáctico:: Error:: Se esperaba L_ENTERO." << std::endl;
    delete nodo;
    return 0;
   }
   nodo->objeto = _analizador_lexico->ilexema();
   _analizador_lexico->leer();
   if(_analizador_lexico->token() != ']')
   {
    std::cout << "Analizador sintáctico:: Error:: Se esperaba ']'." << std::endl;
    delete nodo;
    return 0;
   }
   _analizador_lexico->leer();
   if(_analizador_lexico->token() != '.')
   {
    std::cout << "Analizador sintáctico:: Error:: Se esperaba '.'." << std::endl;
    delete nodo;
    return 0;
   }
   _analizador_lexico->leer();
   if(_analizador_lexico->token() != L_PARAMETROS)
   {
    std::cout << "Analizador sintáctico:: Error:: Se esperaba 'parametros'." << std::endl;
    delete nodo;
    return 0;
   }
   _analizador_lexico->leer();
   if(_analizador_lexico->token() != '{')
   {
    std::cout << "Analizador sintáctico:: Error:: Se esperaba '{'." << std::endl;
    delete nodo;
    return 0;
   }
   nodo->parametro = _asdr_parametro();
   _analizador_lexico->leer();
   if(_analizador_lexico->token() != '}')
   {
    std::cout << "Analizador sintáctico:: Error:: Se esperaba '}'." << std::endl;
    delete nodo;
    return 0;
   }
  }
  else
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba 'retorno' o 'objeto'." << std::endl;
   delete nodo;
   return 0;
  }

  nodo->accion = _asdr_accion();
  return nodo;
 }
 else if(_analizador_lexico->token() == '}')
 {
  delete nodo;
  _analizador_lexico->rebobinar();
  return 0;
 }
 else
 {
  std::cout << "Analizador sintáctico:: Error:: Se esperaba 'ventana' o '}'." << std::endl;
  delete nodo;
  return 0;
 }
 return 0;
}

/*******************************************************************************
*
* Método _asdr_evento()
*
* Gramática:
* EVENTOS => T_EVENTO '{' ACCIONES '}' EVENTOS
*         => *
*
*******************************************************************************/
nodo_evento * c_sintactico::_asdr_evento()
{
 nodo_evento * nodo = new nodo_evento();

 _analizador_lexico->leer();
 if(_e_tokens.count(_analizador_lexico->token()))
 {
  nodo->t_evento = _analizador_lexico->token();
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != '{')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba '{'." << std::endl;
   delete nodo;
   return 0;
  }
  nodo->accion = _asdr_accion();
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != '}')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba '}'." << std::endl;
   delete nodo;
   return 0;
  }
  _analizador_lexico->leer();
  if(_e_tokens.count(_analizador_lexico->token()))
  {
   _analizador_lexico->rebobinar();
   nodo->evento = _asdr_evento();
   return nodo;
  }
  _analizador_lexico->rebobinar();
  return nodo;
 }
 else if(_analizador_lexico->token() == '}')
 {
  _analizador_lexico->rebobinar();
  delete nodo;
  return 0;
 }
 else
 {
  std::cout << "Analizador sintáctico:: Error:: Se esperaba evento." << std::endl;
  delete nodo;
  return 0;
 }
}

/*******************************************************************************
*
* Método _asdr_objeto()
*
* Gramática:
* OBJETOS => T_OBJETO '(' PARAMETROS ')' ( '{' EVENTOS '}' )? OBJETOS
*         => *
*
*******************************************************************************/
nodo_objeto * c_sintactico::_asdr_objeto()
{
 nodo_objeto * nodo = new nodo_objeto();

 _analizador_lexico->leer();
 if(_ov_tokens.count(_analizador_lexico->token()))
 {
  nodo->t_objeto = _analizador_lexico->token();

  _analizador_lexico->leer();
  if(_analizador_lexico->token() != '(')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba '('." << std::endl;
   delete nodo;
   return 0;
  }
  _analizador_lexico->leer();
  if(_i_tokens.count(_analizador_lexico->token()) ||
     _b_tokens.count(_analizador_lexico->token()) ||
     _3f_tokens.count(_analizador_lexico->token()))
  {
   _analizador_lexico->rebobinar();
   nodo->parametro = _asdr_parametro();
  }
  else
   _analizador_lexico->rebobinar();

  _analizador_lexico->leer();
  if(_analizador_lexico->token() != ')')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba ')'." << std::endl;
   delete nodo;
   return 0;
  }
 }
 else
 {
  std::cout << "Analizador sintáctico:: Error:: Se esperaba un objeto." << std::endl;
  delete nodo;
  return 0;
 }

 _analizador_lexico->leer();
 if(_analizador_lexico->token() != '{' )
 {
  _analizador_lexico->rebobinar();
 }
 else
 {
  nodo->evento = _asdr_evento();
  _analizador_lexico->leer();
  if(_analizador_lexico->token() != '}')
  {
   std::cout << "Analizador sintáctico:: Error:: Se esperaba un '}'." << std::endl;
   delete nodo;
   return 0;
  }
 }

 _analizador_lexico->leer();
 if(_analizador_lexico->token() != '}')
 {
  _analizador_lexico->rebobinar();
  nodo->objeto = _asdr_objeto();
 }
 else
  _analizador_lexico->rebobinar();
 return nodo;
}

/*******************************************************************************
*
* Método _asdr_ventana()
*
* Gramática:
* VENTANA => '(' PARAMETROS ')' '{' OBJETOS '}'
*
*******************************************************************************/
nodo_ventana * c_sintactico::_asdr_ventana()
{
 nodo_ventana * nodo = new nodo_ventana();

 // Lectura de '(' PARAMETROS ')'
 _analizador_lexico->leer();
 if(_analizador_lexico->token() != '(')
 {
  std::cout << "Analizador sintáctico:: Error:: Se esperaba '('." << std::endl;
  delete nodo;
  return 0;
 }
 _analizador_lexico->leer();
 if(_i_tokens.count(_analizador_lexico->token()) ||
    _b_tokens.count(_analizador_lexico->token()) ||
    _3f_tokens.count(_analizador_lexico->token()))
 {
  _analizador_lexico->rebobinar();
  nodo->parametro = _asdr_parametro();
 }
 else
  _analizador_lexico->rebobinar();

 _analizador_lexico->leer();
 if(_analizador_lexico->token() != ')')
 {
  std::cout << "Analizador sintáctico:: Error:: Se esperaba ')'." << std::endl;
  delete nodo;
  return 0;
 }

 // Lectura de '{' OBJETOS '}'
 _analizador_lexico->leer();
 if(_analizador_lexico->token() != '{')
 {
  std::cout << "Analizador sintáctico:: Error:: Se esperaba '{'." << std::endl;
  delete nodo;
  return 0;
 }

 _analizador_lexico->leer();
 if(_ov_tokens.count(_analizador_lexico->token()))
 {
  _analizador_lexico->rebobinar();
  nodo->objeto = _asdr_objeto();
 }
 else
  _analizador_lexico->rebobinar();

 _analizador_lexico->leer();
 if(_analizador_lexico->token() != '}')
 {
  std::cout << "Analizador sintáctico:: Error:: Se esperaba '}'." << std::endl;
  delete nodo;
  return 0;
 }
 return nodo;
}

/*******************************************************************************
*
* Método _asdr_ventanas()
*
* Gramática:
* VENTANAS => L_VENTANA VENTANA
*          => EOF
*
*******************************************************************************/
nodo_ventanas * c_sintactico::_asdr_ventanas()
{
 nodo_ventanas * nodo = new nodo_ventanas();
 _analizador_lexico->leer();
 // Para VENTANAS => L_VENTANA VENTANA
 if(_analizador_lexico->token() == L_VENTANA)
 {
  nodo->ventana = _asdr_ventana();
  nodo->ventanas = _asdr_ventanas();
  return nodo;
 }
 // Para VENTANAS => EOF
 if(_analizador_lexico->token() == L_FIN)
  return 0;
 // En otro caso se ha producido un error de análisis.
 std::cout << "Analizador sintáctico:: Error:: Se esperaba 'ventana' o 'EOF'." << std::endl;
 delete nodo;
 return 0;
}

/*********************************************************
*
* Fin del analizador sintáctico descendente recursivo.
*
*********************************************************/

/*******************************************************************************
*
* Constructor
* Creo el analizador léxico y lleno los sets con los tokens.
*
*******************************************************************************/
c_sintactico::c_sintactico(const GLbyte * f)
{
 _analizador_lexico = new c_lexico(f);
 _i_tokens.insert(L_ID);
 _i_tokens.insert(L_X);
 _i_tokens.insert(L_Y);
 _i_tokens.insert(L_ANCHO);
 _i_tokens.insert(L_ALTO);
 _i_tokens.insert(L_IMG);
 _i_tokens.insert(L_FACTORTEXTO);
 _i_tokens.insert(L_GROSORTEXTO);

 _b_tokens.insert(L_VISIBLE);
 _b_tokens.insert(L_MOVIL);

 _3f_tokens.insert(L_COLOR);
 _3f_tokens.insert(L_BCOLOR);
 _3f_tokens.insert(L_TCOLOR);

 _s_tokens.insert(L_TEXTO);

 _ov_tokens.insert(L_OBOTON);
 _ov_tokens.insert(L_OIMAGEN);
 _ov_tokens.insert(L_OTEXTO);
 _ov_tokens.insert(L_OCASILLERO);
 _ov_tokens.insert(L_OTEXTOEDITABLE);
 _ov_tokens.insert(L_OSELECTOR);
 _ov_tokens.insert(L_OSELECTORFICHEROS);

 _e_tokens.insert(L_ECLICK);
 _e_tokens.insert(L_EBLUR);
}

/*******************************************************************************
*
* Método procesar()
* Inicio el análisis.
*
*******************************************************************************/
nodo_ventanas * c_sintactico::procesar()
{
 return _asdr_ventanas();
}
