/*
   Copyright (C) 2007 - 2009 by Roberto García Carvajal <roberpot@gmail.com>
   Part of libglsws (http://magiccards.wordpress.com)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3
   or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/


#include <GL/gl.h>
#include <set>
#include <string>
#include "c_lexico.h"

#ifndef _C_SINTACTICO_
#define _C_SINTACTICO_

using namespace std;

/*******************************************************************************
*
* Definiciones de los nodos.
*
* Para construir el árbol de análisis recursivo descendente.
*
*******************************************************************************/
struct nodo_parametro
{
 nodo_parametro();
 GLuint t_parametro;
 GLuint i_valor;
 string s_valor;
 bool b_valor;
 GLfloat fv3_valor[3];
 nodo_parametro * parametro;
};

struct nodo_accion
{
 nodo_accion();
 nodo_accion * accion;
 nodo_parametro * parametro;
 GLuint ventana;
 GLuint objeto;
 bool datos;
};

struct nodo_evento
{
 nodo_evento();
 GLuint t_evento;
 nodo_accion * accion;
 nodo_evento * evento;
};

struct nodo_objeto
{
 nodo_objeto();
 GLuint t_objeto;
 nodo_parametro * parametro;
 nodo_evento * evento;
 nodo_objeto * objeto;
};

struct nodo_ventana
{
 nodo_ventana();
 nodo_parametro * parametro;
 nodo_objeto * objeto;
};

struct nodo_ventanas
{
 nodo_ventanas();
 nodo_ventana * ventana;
 nodo_ventanas * ventanas;
};

/*******************************************************************************
*
* Clase c_sintáctico.
*
* Analizador sintáctico.
*
*******************************************************************************/
class c_sintactico
{
 public:
  /// Constructor. Recibe un nombre de fichero.
  c_sintactico(const GLbyte * f);
  /// Función que analiza sintácticamente el fichero.
  nodo_ventanas * procesar();
 private:
  /// Función del analizador sintáctico recursivo que analiza un parámetro.
  nodo_parametro * _asdr_parametro();
  /// Función del analizador sintáctico recursivo que analiza una acción.
  nodo_accion * _asdr_accion();
  /// Función del analizador sintáctico recursivo que analiza un evento.
  nodo_evento * _asdr_evento();
  /// Función del analizador sintáctico recursivo que analiza un objeto.
  nodo_objeto * _asdr_objeto();
  /// Función del analizador sintáctico recursivo que analiza un conjunto de ventanas.
  nodo_ventanas * _asdr_ventanas();
  /// Función del analizador sintáctico recursivo que analiza una ventana.
  nodo_ventana * _asdr_ventana();
  /// Analizador léxico interno.
  c_lexico * _analizador_lexico;
  /// Conjunto de tokens para los que se esperan parámetros enteros.
  set<GLuint> _i_tokens;
  /// Conjunto de tokens para los que se esperan parámetros booleanos.
  set<GLuint> _b_tokens;
  /// Conjunto de tokens para los que se esperan parámetros strings.
  set<GLuint> _s_tokens;
  /// Conjunto de tokens para los que se esperan parámetros flotantes (3).
  set<GLuint> _3f_tokens;
  /// Conjunto de tokens que son objetos de una ventana.
  set<GLuint> _ov_tokens;
  /// Conjunto de tokens que son eventos.
  set<GLuint> _e_tokens;
};

#endif
