/*
   Copyright (C) 2007 - 2009 by Roberto García Carvajal <roberpot@gmail.com>
   Part of libglsws (http://magiccards.wordpress.com)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3
   or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/

#include <GL/gl.h>
#include <FTGL/ftgl.h>
#include <map>
#include <list>
#include <vector>
#include "c_ventana.h"
#include "c_sintactico.h"
#include "c_oventana.h"
#include "c_ventana.h"
#include "c_esdatos.h"

using namespace std;

#ifndef _C_SVENTANA_
#define _C_SVENTANA_

/*******************************************************************************
*
* Definiciones de las banderas de estado del sistema de ventanas.
*
*******************************************************************************/
#define SV_CLICKABAJO 0x01

/*******************************************************************************
*
* Clase c_sventana
*
* Sistema de ventanas.
*
*******************************************************************************/
class c_sventana
{
 public:
  /// Constructor.
  c_sventana();
  /// Crea una nueva ventana.
  GLint nventana(GLint id);
  /// Devuelve una ventana.
  c_ventana * ventana(GLint id);
  /// Carga datos en una ventana.
  void cargar(GLint i,c_datos d);
  /// Elimina una ventana.
  void eventana(GLint id);
  /// Hace comprobaciones cuando se hace click.
  bool ci_abajo(GLfloat x,GLfloat y);
  /// Hace comprobaciones cuando se levanta el click.
  void ci_arriba(GLint x,GLint y);
  /// Procesa movimientos de raton.
  void c_mover(GLfloat x,GLfloat y);
  /// Renderiza todas las ventana.
  void dibujar();
  /// Parsea un fichero para interpretar las declaraciones de las ventanas.
  bool parsear(const GLbyte * f);
  /// ???
  void texturas(vector<GLuint> t) { _texturas = t; }
  /// Envía un carácter a la ventana que tenga el foco.
  void recogercaracter(GLubyte c);
  /// Borra un carácter de la ventana que tenga el foco.
  void borrarcaracter();
  /// Asigna la fuente en uso.
  void fuente(FTGLTextureFont * f) { _fuente = f; }
  /// Devuelve los datos de una ventana.
  c_datos datos(GLuint i);
 private:
  /// Genera todo el sistema de ventanas.
  nodo_ventanas * generar_ventanas(nodo_ventanas * ventanas);
  /// Genera una ventana.
  nodo_ventana * nueva_ventana(nodo_ventana * ventana);
  /// Genera un parámetro.
  nodo_parametro * nuevo_parametro(nodo_parametro * parametro);
  /// Genera un parámetro de un objeto.
  nodo_parametro * nuevo_objparametro(nodo_parametro * parametro);
  /// Genera un objeto.
  nodo_objeto * nuevo_objeto(nodo_objeto * objeto);
  /// Genera un evento.
  nodo_evento * nuevo_evento(nodo_evento * evento);
  /// Genera una acción.
  nodo_accion * nueva_accion(nodo_accion * accion);
  /// Genera un parámetro de una acción.
  nodo_parametro * nuevo_accparametro(nodo_parametro * parametro);
  GLuint _RBE;                      /// Registro de ventanas.
  GLint _id_foco,                   /// Ventana con el foco.
        _x_foco,                    /// Coordenada horizontal de la ventana que tiene el foco donde se hizo click.
        _y_foco,                    /// Coordenada vertical de la ventana que tiene el foco donde se hizo click.
        _c_ventana_id,              /// uid de la ventana que se está creando.
        _c_oventana_id;             /// uid del objeto de ventana que se está creando.
  map<GLint,c_ventana *> _ventanas; /// Conjunto de ventanas.
  c_oventana * _oventana;           /// Objeto de ventana que se está creando.
  c_ventana * _ventana;             /// Ventana que se está creando.
  list<GLint> _vordenadas;          /// Lista ordenada de las ventanas según los clicks que se han hecho.
  vector<GLuint> _texturas;         /// Vector de texturas.
  FTGLTextureFont * _fuente;        /// Fuente que se usará para el texto.
  vector<c_parametro> _eaparametros;/// Vector de parámetros.
  GLuint _tevento;                  /// Último evento.
  map<GLuint,c_datos> _datos;        /// Datos generados por las ventanas.
};

#endif
